'use strict';

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

require('regenerator-runtime/runtime');

var _promises = require('../test-utils/promises');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();
_chai2.default.use(_sinonChai2.default);

var token = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWV9.TJVA95OrM7E2cBab30RMHrHDcEfxjoYZgeFONFh7HgQ';
var decodedToken = {
  "sub": "1234567890",
  "name": "John Doe",
  "admin": true
};

global.window = global;
var localStorage = window.localStorage = {
  items: {},
  getItem: function getItem(name) {
    return localStorage.items[name];
  },
  setItem: function setItem(name, value) {
    return localStorage.items[name] = value;
  },
  removeItem: function removeItem(name) {
    return delete localStorage.items[name];
  }
};

window.addEventListener = function () {};

window.location = {
  // sinon relies on location.protocol if window is defined
  // in /sinon/lib/sinon/util/fake_server.js
  protocol: 'http',
  // For messaging checks
  origin: 'origin'
};

// Miminal options necessary for constructor to init, skipping non-specific tests
var minOpts = {
  apolloClient: {
    query: function query() {
      return {
        then: function then() {}
      };
    } // i.e. skip / no-op
  }
};

describe('ApolloPassport - client', function () {

  describe('- constructor', function () {

    it('instantiates correctly', function () {
      var ap = new _index2.default(minOpts);
      ap.apolloClient.should.equal(minOpts.apolloClient);
    });

    it('runs assertToken if a token exists in localStorage', function (done) {
      localStorage.setItem('apToken', token);
      var ap = new _index2.default(minOpts);
      ap.assertToken = function () {
        ap._token.should.equal(token);
        done();
      };
    });
  });

  describe('- internal', function () {

    describe('_runDiscovery', function () {

      it('throws on errors', function () {
        var apolloClient = {
          query: (0, _promises.FakeResolve)({
            errors: []
          })
        };
        (function () {
          new _index2.default({ apolloClient: apolloClient });
        }).should.throw();
      });

      it('sets .discovered, builds .open', function () {
        var discoveryData = {
          ROOT_URL: 'ROOT_URL',
          authPath: 'ap-auth',
          services: [{
            type: 'oauth',
            name: 'footbook',
            urlStart: 'http://my.service/oauth/auth'
          }, {
            type: 'oauth2',
            name: 'meower',
            urlStart: 'http://my.service/oauth/auth'
          }]
        };
        var apolloClient = {
          query: (0, _promises.FakeResolve)({ data: { apDiscovery: discoveryData } })
        };
        var ap = new _index2.default({ apolloClient: apolloClient });

        ap.discovered.should.equal(discoveryData);

        ap.discovered.services[0].open.should.exist;
        ap.discovered.services[1].open.should.exist;
      });
    });
  });

  describe('- messaging', function () {

    describe('receiveMessage()', function () {
      var ap = new _index2.default(minOpts);

      it('skips irrelevent messages', function () {
        ap.loginComplete = _sinon2.default.spy();

        ap.receiveMessage({ origin: 'not-origin', data: "apolloPassport {}" });
        ap.loginComplete.should.not.have.been.called;

        ap.receiveMessage({ origin: window.location.origin, data: {} });
        ap.loginComplete.should.not.have.been.called;
      });

      it('calls loginComplete', function () {
        ap.loginComplete = _sinon2.default.spy();
        var data = { type: 'loginComplete', key: 'key' };
        var dataStr = 'apolloPassport ' + JSON.stringify(data);
        ap.receiveMessage({ origin: window.location.origin, data: dataStr });
        ap.loginComplete.should.have.been.calledWith(data, data.key);
      });

      it('throws on invalid key', function () {
        var data = { type: 'unknown' };
        var dataStr = 'apolloPassport ' + JSON.stringify(data);
        (function () {
          ap.receiveMessage({ origin: window.location.origin, data: dataStr });
        }).should.throw();
      });
    });
  });

  describe('- modules', function () {

    it('can be used', function () {
      var ap = new _index2.default(minOpts);
      var TestStrategy = function TestStrategy() {};
      ap.use('test', TestStrategy);
      ap.strategies.test.should.be.an.instanceof(TestStrategy);
    });

    it('can extend the instance', function () {
      var ap = new _index2.default(minOpts);
      var methods = { a: {} };
      ap.extendWith(methods);
      ap.a.should.equal(methods.a);
    });
  });

  describe('- actions -', function () {

    it('loginStart()', function () {});

    describe('loginComplete()', function () {

      it('handles errors', function () {
        var ap = new _index2.default(minOpts);
        var result = {
          errors: []
        };

        ap.loginComplete(result, 'unused');
        // Nothing to test for for now.
      });

      it('sets state on success', function () {
        var ap = new _index2.default(minOpts);
        var result = {
          data: {
            passportLoginEmail: { token: token }
          }
        };

        ap.loginComplete(result, 'passportLoginEmail');
        ap.getState().should.deep.equal({
          data: decodedToken,
          verified: true,
          error: null
        });
      });

      it('sets state on error', function () {
        var ap = new _index2.default(minOpts);
        var result = {
          data: {
            passportLoginEmail: { error: "something" }
          }
        };

        ap.loginComplete(result, 'passportLoginEmail');
        ap.getState().should.deep.equal({
          data: {},
          verified: false,
          error: "something"
        });
      });
    });

    it('logout() clears state & local storage', function () {
      var ap = new _index2.default(minOpts);

      localStorage.setItem('apToken', 'a');
      ap.setState({ data: decodedToken, verified: true, error: null });

      ap.logout();

      var state = ap.getState();
      state.data.should.deep.equal({});
      state.verified.should.be.false;
    });
  });

  describe('- state', function () {

    it('allows removal subscribers', function () {
      var ap = new _index2.default(minOpts);
      var test = function test() {};

      ap.subscribe(test);
      ap.unsubscribe(test);
      ap._subscribers.has(test).should.be.false;
    });

    // subscribe+emitState tested in redux suite below.
  });

  describe('- optional redux support', function () {

    describe('- reducer', function () {

      var ap = new _index2.default(minOpts);
      var reducer = ap.reducer();
      var state = {};

      it('changes state on APOLLO_PASSPORT_UPDATE', function () {
        reducer({}, {
          type: 'APOLLO_PASSPORT_UPDATE',
          state: state
        }).should.equal(state);
      });

      it('should return given state on non matching action', function () {
        reducer(state, {}).should.equal(state);
      });

      it('should return getState as initial value', function () {
        reducer(undefined, {}).should.equal(ap.getState());
      });
    });

    describe('- middleware', function () {

      it('keeps ref to store and returns identity middleware', function () {

        var ap = new _index2.default(minOpts);
        var middleware = ap.middleware();
        var store = {};
        var m = middleware(store);
        ap.store.should.equal(store);

        // test the identity middleware
        var action = {};
        m(function () {
          return action.test = 1;
        })(action);
        action.test.should.equal(1);
      });

      it('updates redux state on apolloPassport state changes', function (done) {

        var ap = new _index2.default(minOpts);
        var middleware = ap.middleware();
        var nextState = { data: {}, verified: true, error: null };
        var store = {
          dispatch: function dispatch(action) {
            action.type.should.equal('APOLLO_PASSPORT_UPDATE');
            action.state.should.equal(nextState);
            done();
          }
        };

        middleware(store);
        ap.setState(nextState);
      });
    });
  });
});