'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _promises = require('./test-utils/promises');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var should = _chai2.default.should();
_chai2.default.use(_sinonChai2.default);

var jwtSecret = 'xxx';
var requiredOptions = function requiredOptions() {
  return {
    jwtSecret: jwtSecret,
    db: { createUser: true, fetchUserByEmail: true },
    ROOT_URL: 'http://localhost:3000/'
  };
};

describe('apollo-passport', function () {

  describe('default helpers', function () {

    describe('defaultMapUserToJWTProps', function () {
      it('maps from a user record to an object with a userId field', function () {
        var user = { id: 1 };
        (0, _index.defaultMapUserToJWTProps)(user).should.deep.equal({ userId: 1 });
      });
    });

    describe('defaultCreateTokenFromUser', function () {
      it('x', function () {
        var context = {
          mapUserToJWTProps: function mapUserToJWTProps(user) {
            return { userId: user.id };
          },

          jwtSecret: jwtSecret
        };

        var user = { id: 1 };
        var token = _index.defaultCreateTokenFromUser.call(context, user);

        var data = _jsonwebtoken2.default.decode(token);
        delete data.iat;
        delete data.exp;
        data.should.deep.equal(context.mapUserToJWTProps(user));
      });
    });
  });

  describe('constructor()', function () {

    it('requires a valid db driver instance', function () {
      var options = requiredOptions();

      (function () {
        new _index2.default(options);
      }).should.not.throw();

      (function () {
        delete options.db;
        new _index2.default(options);
      }).should.throw();

      (function () {
        options.db = {};
        new _index2.default(options);
      }).should.throw();
    });

    it('throws on no jwtSecret', function () {
      var options = requiredOptions();
      delete options.jwtSecret;
      (function () {
        new _index2.default(options);
      }).should.throw();
    });

    describe('ROOT_URL', function () {

      it('throws on no ROOT_URL', function () {
        var options = requiredOptions();
        delete options.ROOT_URL;

        (function () {
          new _index2.default(options);
        }).should.throw();
      });

      it('accepts a ROOT_URL global', function () {
        var options = requiredOptions();
        delete options.ROOT_URL;
        global.ROOT_URL = 'ROOT_URL/';
        var ap = new _index2.default(options);
        ap.ROOT_URL.should.equal(ROOT_URL);
        delete global.ROOT_URL;
      });

      it('accepts process.env.ROOT_URL', function () {
        var options = requiredOptions();
        delete options.ROOT_URL;
        process.env.ROOT_URL = 'ROOT_URL/';
        var ap = new _index2.default(options);
        ap.ROOT_URL.should.equal(process.env.ROOT_URL);
        delete process.env.ROOT_URL;
      });

      it('appends trailing / if one does not exist', function () {
        var options = requiredOptions();
        options.ROOT_URL = 'ROOT_URL';
        var ap = new _index2.default(options);
        ap.ROOT_URL.should.equal('ROOT_URL/');
      });
    });

    it('strips leading "/" from authPath', function () {
      var options = requiredOptions();
      options.authPath = '/hello';
      var ap = new _index2.default(options);
      ap.authPath.should.equal('hello');
    });
  });

  describe('use()', function () {

    var ap = new _index2.default(requiredOptions());
    ap.passport = {
      use: function use() {}
    };
    ap.require = function (strategy, module) {
      switch (module) {
        case 'index':
          return function () {};
        case 'resolvers':
          return { RootMutation: {} };
        case 'schema':
          return [''];
        case 'verify':
          return function () {
            return { _defaultVerify: true, self: this };
          };
        case 'defaultOptions':
          return { _defaultOptions: true };

        /* just to help with writing tests */
        /* istanbul ignore next */
        default:
          throw new Error("No " + module);
      }
    };

    it('accepts an AugmentedStrategy', function () {
      var AugmentedStrategy = function AugmentedStrategy(apolloPassport, options) {
        _classCallCheck(this, AugmentedStrategy);

        this.ap = apolloPassport;

        function Strategy() {}
        this.strategy = new Strategy();

        this.resolvers = {};
        this.schema = [''];
      };

      AugmentedStrategy.__isAugmented = true;

      ap.use('augmented', AugmentedStrategy);

      ap.strategies.augmented.should.be.an.instanceOf(AugmentedStrategy);
    });

    it('calls passport.use(new Strategy(options, boundVerify))', function () {
      var options = {};
      var verify = function verify() {
        return { self: this };
      };

      function FakeStrategy(options, boundVerify) {
        options.should.equal(options);
        boundVerify().self.should.equal(ap);
      }
      ap.use('local', FakeStrategy, options, verify);
    });

    it('accepts uses default options for missing options / verify', function () {
      function FakeStrategy1(options, boundVerify) {
        options.should.deep.equal({ _defaultOptions: true });
        boundVerify().should.deep.equal({ self: ap });
      }
      var verify = function verify() {
        return { self: this };
      };
      ap.use('local', FakeStrategy1, verify);

      function FakeStrategy2(options, boundVerify) {
        options.should.deep.equal({ _defaultOptions: true });
        boundVerify().should.deep.equal({ _defaultVerify: true, self: ap });
      }
      ap.use('local', FakeStrategy2);
    });

    it('sets default callbackURL on oauth methods', function () {
      function FakeStrategy(options) {
        options.callbackURL.should.equal(ap.authUrlRoot + '/fake');
      }
      ap.use('oauth2:fake', FakeStrategy, {});
    });

    it('calls passport.authenticate if a scope is given', function () {
      var apOptions = requiredOptions();
      var authenticate = _sinon2.default.spy();
      apOptions.passport = {
        use: function use() {},
        authenticate: authenticate };

      var ap = new _index2.default(apOptions);
      var scopeOptions = { scope: ['a'] };
      ap.use('fake', function () {}, scopeOptions, function () {});

      authenticate.should.have.been.calledWith('fake', scopeOptions);
    });
  });

  describe('extendsWith()', function () {
    it('extends the context', function () {
      var extensions = { a: 1 };
      var ap = new _index2.default(requiredOptions());
      ap.extendWith(extensions);
      ap.a.should.equal(1);
    });
  });

  /////////////////////////////
  // require() and resolve() //
  /////////////////////////////

  describe('resolve()', function () {
    var ap = new _index2.default(requiredOptions());
    (function () {
      ap.resolve({}); // expects a string
    }).should.throw();
  });

  describe('require()', function () {
    var es5name = './test-utils/sample-module-es5.js';
    var es5 = require(es5name);
    var es6name = './test-utils/sample-module-es6.js';
    var es6 = require(es6name);

    var ap = new _index2.default(requiredOptions());
    ap.resolve = function (module) {
      if (module.endsWith('es6')) return es6name;
      if (module.endsWith('es5')) return es5name;
      return null;
    };

    it('throws on unfound modules', function () {
      (function () {
        ap.require('non-existant', 'non-existant');
      }).should.throw();
    });

    it('returns a loaded, found module', function () {
      ap.require('local', 'es5').should.equal(es5);
      ap.require('local', 'es6').should.equal(es6.default);
    });
  });

  ///////////
  // Users //
  ///////////

  describe('users', function () {

    describe('createUser', function () {

      it('calls db.createUser and returns the userId', function _callee() {
        var desiredUserId, options, ap, userId;
        return regeneratorRuntime.async(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                desiredUserId = '1';
                options = requiredOptions();

                options.db.createUser = function () {
                  return Promise.resolve(desiredUserId);
                };

                ap = new _index2.default(options);
                _context.next = 6;
                return regeneratorRuntime.awrap(ap.createUser({}));

              case 6:
                userId = _context.sent;


                userId.should.equal(desiredUserId);

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, null, undefined);
      });
    });
  });

  //////////////////////
  // GraphQL & Apollo //
  //////////////////////

  describe('schema()', function () {
    it('returns this._schema', function () {
      var ap = new _index2.default(requiredOptions());
      ap.schema().should.equal(ap._schema);
    });
  });

  describe('resolvers()', function () {
    it('returns a bound version of this._resolvers', function () {
      var ap = new _index2.default(requiredOptions());
      ap._resolvers = { a: 1 };
      ap._bindRootQueriesAndMutations = function (x) {
        return _extends({}, x, { _bound: 1 });
      };
      ap.resolvers().should.deep.equal(_extends({}, ap._resolvers, { _bound: 1 }));
    });
  });

  describe('wrapOptions() func', function () {

    var options = { a: 1 };
    var ap = new _index2.default(requiredOptions());
    var wrapper = ap.wrapOptions(options); // <-- returns async function!
    var reqAuthBearer = function reqAuthBearer(token) {
      return { headers: { authorization: 'Bearer ' + token } };
    };

    it('returns the original options if no token in headers', function _callee2() {
      return regeneratorRuntime.async(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return regeneratorRuntime.awrap(wrapper({ headers: {} }));

            case 2:
              _context2.t0 = options;

              _context2.sent.should.equal(_context2.t0);

              _context2.next = 6;
              return regeneratorRuntime.awrap(wrapper({ headers: {
                  authorization: 'Some unrecognized type'
                } }));

            case 6:
              _context2.t1 = options;

              _context2.sent.should.equal(_context2.t1);

              _context2.next = 10;
              return regeneratorRuntime.awrap(wrapper({ headers: {
                  authorization: 'Bearer ' // empty token
                } }));

            case 10:
              _context2.t2 = options;

              _context2.sent.should.equal(_context2.t2);

            case 12:
            case 'end':
              return _context2.stop();
          }
        }
      }, null, undefined);
    });

    // e.g. invalid token, jwt expired
    it('adds a jwtError to the context if one occurred', function _callee3() {
      var token, result;
      return regeneratorRuntime.async(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              token = _jsonwebtoken2.default.sign({ userId: 1 }, 'a different secret');
              _context3.next = 3;
              return regeneratorRuntime.awrap(wrapper(reqAuthBearer(token)));

            case 3:
              result = _context3.sent;

              should.not.exist(result.context.auth);
              result.context.authError.name.should.equal('JsonWebTokenError');
              result.context.authError.message.should.equal('invalid signature');

            case 7:
            case 'end':
              return _context3.stop();
          }
        }
      }, null, undefined);
    });

    it('adds the decoded value of a valid token to the context', function _callee4() {
      var token, result;
      return regeneratorRuntime.async(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              token = _jsonwebtoken2.default.sign({ userId: 1 }, jwtSecret);
              _context4.next = 3;
              return regeneratorRuntime.awrap(wrapper(reqAuthBearer(token)));

            case 3:
              result = _context4.sent;

              result.context.auth.userId.should.equal(1);

            case 5:
            case 'end':
              return _context4.stop();
          }
        }
      }, null, undefined);
    });
  });

  describe('_bindRootQueriesAndMutations()', function () {
    it('binds functions in RootMutations/RootQueries keys (only)', function () {
      var resolvers = {
        x: function x() {
          return this;
        },

        RootMutation: {
          x: function x() {
            return this;
          }
        },
        RootQuery: {
          x: function x() {
            return this;
          }
        }
      };

      var ap = new _index2.default(requiredOptions());
      var bound = ap._bindRootQueriesAndMutations(resolvers);

      bound.x().should.not.equal(ap);
      bound.RootMutation.x().should.equal(ap);
      bound.RootQuery.x().should.equal(ap);
    });
  });

  /////////////////
  // Middlewares //
  /////////////////

  describe('middleware', function () {

    // honestly we should just skip this, or at least validate html
    it('popupScript', function () {
      var data = { a: 1 };
      var ap = new _index2.default(requiredOptions());
      var out = ap.popupScript(data);
      var matches = out.match(/postMessage\('apolloPassport (.*)', window.location/);
      JSON.parse(matches[1]).should.deep.equal(data);
    });

    describe('expressMiddleware()', function () {

      it('passes content', function () {
        var ap = new _index2.default(requiredOptions());
        ap.apAuthenticate = (0, _promises.FakeResolve)('data');
        ap.popupScript = function () {
          return 'popupScript';
        };

        var middleware = ap.expressMiddleware();
        var req = { url: '/ap-auth/facebook/callback' };
        var res = {
          setHeader: function setHeader() {},
          end: _sinon2.default.spy() };

        middleware(req, res);
        res.end.should.have.been.calledWith('popupScript', 'utf8');
      });

      it('handles errors', function () {
        var ap = new _index2.default(requiredOptions());
        ap.apAuthenticate = (0, _promises.FakeReject)(new Error('error'));

        var middleware = ap.expressMiddleware();
        var req = { url: '/ap-auth/facebook/callback' };
        var res = { status: _sinon2.default.spy(), end: _sinon2.default.spy() };

        middleware(req, res);
        res.status.should.have.been.calledWith(500, 'Internal server error');
        res.end.should.have.been.called;
      });
    });
  });
});