'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

require('regenerator-runtime/runtime');

var _verify = require('./verify');

var _verify2 = _interopRequireDefault(_verify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();
_chai2.default.use(_sinonChai2.default);

describe('apollo-passport - strategies - oauth2 - verify', function () {

  it('warns if strategy != provider', function () {
    _sinon2.default.spy(console, 'warn');

    var context = {
      db: {
        // Short circuit everything else for this test.

        fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail() {
          return {
            then: function then() {
              return {
                catch: function _catch() {}
              };
            }
          };
        }
      }
    };

    _verify2.default.call(context, "strategy", null, null, { provider: 'not-strategy' });

    console.warn.should.have.been.called.once;
    console.warn.restore();
  });

  it('correctly infers and sets values before calling fetchUserByServiceIdOrEmail', function () {
    var context = {
      db: {
        // Short circuit everything else for this test.

        fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail() {
          return {
            then: function then() {
              return {
                catch: function _catch() {}
              };
            }
          };
        }
      }
    };
    _sinon2.default.spy(context.db, 'fetchUserByServiceIdOrEmail');

    _verify2.default.call(context, "strategy", "accessToken", "refreshToken", {
      id: 'profileId',
      provider: "provider",
      emails: [{ value: "me@me.com" }]
    });

    context.db.fetchUserByServiceIdOrEmail.should.have.been.calledWith("provider", "profileId", "me@me.com");
  });

  describe('correctly handles fetched user data', function () {

    it('creates a user if not returned (without email)', function (done) {
      var profile = { id: 'profileId', provider: 'provider' };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then) {
                _then(null); // i.e. no user found
                return {
                  catch: function _catch() {}
                };
              }
            };
          }
        },
        createUser: function createUser(user) {
          return {
            then: function then(func) {
              func("userId");return {
                catch: function _catch() {}
              };
            }
          };
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        user.should.deep.equal({
          emails: [],
          services: {
            provider: profile
          },
          id: "userId"
        });
        done();
      });
    });

    it('creates a user if not returned (email)', function (done) {
      var profile = {
        id: 'profileId', provider: 'provider',
        emails: [{ value: 'me@me.com' }]
      };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then2) {
                _then2(null); // i.e. no user found
                return {
                  catch: function _catch() {}
                };
              }
            };
          }
        },
        createUser: function createUser(user) {
          return {
            then: function then(func) {
              func("userId");return {
                catch: function _catch() {}
              };
            }
          };
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        user.should.deep.equal({
          emails: [{ address: "me@me.com" }],
          services: {
            provider: profile
          },
          id: "userId"
        });
        done();
      });
    });

    it('adds provider profile if data if it didn\'t exist before', function (done) {
      var profile = { id: 'profileId', provider: 'provider', something: true };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then3) {
                _then3({
                  id: "userId",
                  emails: [{ address: "me@me.com" }],
                  services: {}
                });
                return {
                  catch: function _catch() {}
                };
              }
            };
          },

          assertUserServiceData: _sinon2.default.spy()
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        context.db.assertUserServiceData.should.have.been.calledWith("userId", "provider", profile);

        user.should.deep.equal({
          emails: [{ address: "me@me.com" }],
          services: {
            provider: profile
          },
          id: "userId"
        });

        done();
      });
    });

    it('updates provider profile if data has changed', function (done) {
      var profile = { id: 'profileId', provider: 'provider', something: true };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then4) {
                _then4({
                  id: "userId",
                  emails: [{ address: "me@me.com" }],
                  services: { provider: _extends({}, profile, { something: false }) }
                });
                return {
                  catch: function _catch() {}
                };
              }
            };
          },

          assertUserServiceData: _sinon2.default.spy()
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        context.db.assertUserServiceData.should.have.been.calledWith("userId", "provider", profile);

        user.should.deep.equal({
          id: "userId",
          emails: [{ address: "me@me.com" }],
          services: { provider: profile }
        });

        done();
      });
    });

    it('adds email address if it didn\t exist before', function (done) {
      var profile = {
        id: 'profileId', provider: 'provider',
        emails: [{ value: 'me2@me.com' }]
      };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then5) {
                _then5({
                  id: "userId",
                  emails: [{ address: "me@me.com" }],
                  services: { provider: profile }
                });
                return {
                  catch: function _catch() {}
                };
              }
            };
          },
          assertUserServiceData: function assertUserServiceData() {},

          assertUserEmailData: _sinon2.default.spy()
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        context.db.assertUserEmailData.should.have.been.calledWith("userId", "me2@me.com");

        user.emails.should.deep.equal([{ address: "me@me.com" }, { address: "me2@me.com" }]);

        done();
      });
    });

    it('adds email address if it didn\t exist before (and user had no emails)', function (done) {
      var profile = {
        id: 'profileId', provider: 'provider',
        emails: [{ value: 'me2@me.com' }]
      };
      var context = {
        db: {
          fetchUserByServiceIdOrEmail: function fetchUserByServiceIdOrEmail(provider, id, email) {
            return {
              then: function then(_then6) {
                _then6({ id: "userId", services: { provider: profile } });
                return {
                  catch: function _catch() {}
                };
              }
            };
          },
          assertUserServiceData: function assertUserServiceData() {},

          assertUserEmailData: _sinon2.default.spy()
        }
      };

      _verify2.default.call(context, "provider", "accessToken", null, profile, function (err, user) {
        context.db.assertUserEmailData.should.have.been.calledWith("userId", "me2@me.com");
        user.emails.should.deep.equal([{ address: "me2@me.com" }]);
        done();
      });
    });
  });
});